package com.sun.xml.internal.ws.api.addressing;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.XMLFilterImpl;
import org.xml.sax.ErrorHandler;
import org.xml.sax.ContentHandler;
import javax.xml.soap.SOAPHeader;
import com.sun.xml.internal.stream.buffer.XMLStreamBufferException;
import javax.xml.soap.SOAPException;
import org.w3c.dom.Element;
import javax.xml.soap.SOAPMessage;
import com.sun.xml.internal.ws.util.xml.XMLStreamWriterFilter;
import javax.xml.stream.XMLStreamWriter;
import javax.xml.namespace.QName;
import javax.xml.stream.util.StreamReaderDelegate;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamException;
import javax.xml.ws.WebServiceException;
import com.sun.istack.internal.NotNull;
import com.sun.istack.internal.FinalArrayList;
import com.sun.xml.internal.stream.buffer.XMLStreamBuffer;
import com.sun.xml.internal.ws.message.AbstractHeaderImpl;

final class OutboundReferenceParameterHeader extends AbstractHeaderImpl
{
    private final XMLStreamBuffer infoset;
    private final String nsUri;
    private final String localName;
    private FinalArrayList<OutboundReferenceParameterHeader.Attribute> attributes;
    private static final String TRUE_VALUE = "1";
    private static final String IS_REFERENCE_PARAMETER = "IsReferenceParameter";
    
    OutboundReferenceParameterHeader(XMLStreamBuffer infoset, String nsUri, String localName) {
        this.infoset = infoset;
        this.nsUri = nsUri;
        this.localName = localName;
    }
    
    @NotNull
    public String getNamespaceURI() {
        return this.nsUri;
    }
    
    @NotNull
    public String getLocalPart() {
        return this.localName;
    }
    
    public String getAttribute(String nsUri, String localName) {
        if (this.attributes == null) {
            this.parseAttributes();
        }
        int i = this.attributes.size() - 1;
        while (i >= 0) {
            OutboundReferenceParameterHeader.Attribute a = (OutboundReferenceParameterHeader.Attribute)this.attributes.get(i);
            if (a.localName.equals(localName) && a.nsUri.equals(nsUri)) {
                return a.value;
            }
            --i;
        }
        return null;
    }
    
    private void parseAttributes() {
        try {
            XMLStreamReader reader = this.readHeader();
            reader.nextTag();
            this.attributes = new FinalArrayList();
            boolean refParamAttrWritten = false;
            for (int i = 0; i < reader.getAttributeCount(); ++i) {
                String attrLocalName = reader.getAttributeLocalName(i);
                String namespaceURI = reader.getAttributeNamespace(i);
                String value = reader.getAttributeValue(i);
                if (namespaceURI.equals(AddressingVersion.W3C.nsUri) && attrLocalName.equals("IS_REFERENCE_PARAMETER")) {
                    refParamAttrWritten = true;
                }
                this.attributes.add(new Attribute(namespaceURI, attrLocalName, value));
            }
            if (!refParamAttrWritten) {
                this.attributes.add(new OutboundReferenceParameterHeader.Attribute(AddressingVersion.W3C.nsUri, "IsReferenceParameter", "1"));
            }
        }
        catch (XMLStreamException e) {
            throw new WebServiceException("Unable to read the attributes for {" + this.nsUri + "}" + this.localName + " header", e);
        }
    }
    
    public XMLStreamReader readHeader() throws XMLStreamException {
        return new StreamReaderDelegate(this.infoset.readAsXMLStreamReader()) {
            int state = 0;
            
            @Override
            public int next() throws XMLStreamException {
                return this.check(super.next());
            }
            
            @Override
            public int nextTag() throws XMLStreamException {
                return this.check(super.nextTag());
            }
            
            private int check(int type) {
                switch (this.state) {
                    case 0: {
                        if (type == 1) {
                            this.state = 1;
                            break;
                        }
                        break;
                    }
                    case 1: {
                        this.state = 2;
                        break;
                    }
                }
                return type;
            }
            
            @Override
            public int getAttributeCount() {
                if (this.state == 1) {
                    return super.getAttributeCount() + 1;
                }
                return super.getAttributeCount();
            }
            
            @Override
            public String getAttributeLocalName(int index) {
                if (this.state == 1 && index == super.getAttributeCount()) {
                    return "IsReferenceParameter";
                }
                return super.getAttributeLocalName(index);
            }
            
            @Override
            public String getAttributeNamespace(int index) {
                if (this.state == 1 && index == super.getAttributeCount()) {
                    return AddressingVersion.W3C.nsUri;
                }
                return super.getAttributeNamespace(index);
            }
            
            @Override
            public String getAttributePrefix(int index) {
                if (this.state == 1 && index == super.getAttributeCount()) {
                    return "wsa";
                }
                return super.getAttributePrefix(index);
            }
            
            @Override
            public String getAttributeType(int index) {
                if (this.state == 1 && index == super.getAttributeCount()) {
                    return "CDATA";
                }
                return super.getAttributeType(index);
            }
            
            @Override
            public String getAttributeValue(int index) {
                if (this.state == 1 && index == super.getAttributeCount()) {
                    return "1";
                }
                return super.getAttributeValue(index);
            }
            
            @Override
            public QName getAttributeName(int index) {
                if (this.state == 1 && index == super.getAttributeCount()) {
                    return new QName(AddressingVersion.W3C.nsUri, "IsReferenceParameter", "wsa");
                }
                return super.getAttributeName(index);
            }
            
            @Override
            public String getAttributeValue(String namespaceUri, String localName) {
                if (this.state == 1 && localName.equals("IsReferenceParameter") && namespaceUri.equals(AddressingVersion.W3C.nsUri)) {
                    return "1";
                }
                return super.getAttributeValue(namespaceUri, localName);
            }
        };
    }
    
    public void writeTo(final XMLStreamWriter w) throws XMLStreamException {
        this.infoset.writeToXMLStreamWriter(new XMLStreamWriterFilter(w) {
            private boolean root = true;
            private boolean onRootEl = true;
            
            public void writeStartElement(String localName) throws XMLStreamException {
                super.writeStartElement(localName);
                this.writeAddedAttribute();
            }
            
            private void writeAddedAttribute() throws XMLStreamException {
                if (!this.root) {
                    this.onRootEl = false;
                    return;
                }
                this.root = false;
                this.writeNamespace("wsa", AddressingVersion.W3C.nsUri);
                super.writeAttribute("wsa", AddressingVersion.W3C.nsUri, "IsReferenceParameter", "1");
                return;
            }
            
            public void writeStartElement(String namespaceURI, String localName) throws XMLStreamException {
                super.writeStartElement(namespaceURI, localName);
                this.writeAddedAttribute();
            }
            
            public void writeStartElement(String prefix, String localName, String namespaceURI) throws XMLStreamException {
                boolean prefixDeclared = this.isPrefixDeclared(prefix, namespaceURI);
                super.writeStartElement(prefix, localName, namespaceURI);
                if (!prefixDeclared && !prefix.equals("")) {
                    super.writeNamespace(prefix, namespaceURI);
                }
                this.writeAddedAttribute();
            }
            
            public void writeNamespace(String prefix, String namespaceURI) throws XMLStreamException {
                if (!this.isPrefixDeclared(prefix, namespaceURI)) {
                    super.writeNamespace(prefix, namespaceURI);
                }
            }
            
            public void writeAttribute(String prefix, String namespaceURI, String localName, String value) throws XMLStreamException {
                if (this.onRootEl && namespaceURI.equals(AddressingVersion.W3C.nsUri) && localName.equals("IsReferenceParameter")) {
                    return;
                }
                this.writer.writeAttribute(prefix, namespaceURI, localName, value);
                return;
            }
            
            public void writeAttribute(String namespaceURI, String localName, String value) throws XMLStreamException {
                this.writer.writeAttribute(namespaceURI, localName, value);
            }
            
            private boolean isPrefixDeclared(String prefix, String namespaceURI) {
                return namespaceURI.equals(this.getNamespaceContext().getNamespaceURI(prefix));
            }
        }, true);
    }
    
    public void writeTo(SOAPMessage saaj) throws SOAPException {
        try {
            SOAPHeader header = saaj.getSOAPHeader();
            if (header == null) {
                header = saaj.getSOAPPart().getEnvelope().addHeader();
            }
            Element node = (Element)this.infoset.writeTo(header);
            node.setAttributeNS(AddressingVersion.W3C.nsUri, AddressingVersion.W3C.getPrefix() + ":" + "IsReferenceParameter", "1");
        }
        catch (XMLStreamBufferException e) {
            throw new SOAPException(e);
        }
    }
    
    public void writeTo(ContentHandler contentHandler, ErrorHandler errorHandler) throws SAXException {
        class Filter extends XMLFilterImpl
        {
            private int depth;
            
            Filter() {
                this.depth = 0;
                this.setContentHandler(ch);
            }
            
            @Override
            public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
                if (this.depth++ == 0) {
                    super.startPrefixMapping("wsa", AddressingVersion.W3C.nsUri);
                    if (atts.getIndex(AddressingVersion.W3C.nsUri, "IsReferenceParameter") == -1) {
                        AttributesImpl atts2 = new AttributesImpl(atts);
                        atts2.addAttribute(AddressingVersion.W3C.nsUri, "IsReferenceParameter", "wsa:IsReferenceParameter", "CDATA", "1");
                        atts = atts2;
                    }
                }
                super.startElement(uri, localName, qName, atts);
            }
            
            @Override
            public void endElement(String uri, String localName, String qName) throws SAXException {
                super.endElement(uri, localName, qName);
                int depth = this.depth - 1;
                this.depth = depth;
                if (depth == 0) {
                    super.endPrefixMapping("wsa");
                }
            }
        }
        this.infoset.writeTo(new Filter(contentHandler), errorHandler);
    }
    
    static final class Attribute
    {
        final String nsUri;
        final String localName;
        final String value;
        
        public Attribute(String nsUri, String localName, String value) {
            this.nsUri = fixNull(nsUri);
            this.localName = localName;
            this.value = value;
        }
        
        private static String fixNull(String s) {
            return (s == null) ? "" : s;
        }
    }
}
